(defun c:F2S (/         *error*   clockwise-p         3d-coord->pt-lst
              Triang2dArea        space     loft      ss
              alt       count     coords    p1        p2
              p3        p4        lowest    solid     flst
              cnt       previous  slst      tmp       n
              i
             )

  ;; Redfinition locale de *error*
  (defun *error* (msg)
    (or (= msg "Fonction annule")
        (princ (strcat "\nErreur: " msg))
    )
    (setvar 'solidhist shist)
    (vla-EndUndoMark *acdoc*)
    (princ)
  )

  ;; Clockwise-p Retourne T si les points p1 p2 et p3 tournent dans le sens horaire
  (defun clockwise-p (p1 p2 p3)
    (< (sin (- (angle p1 p3) (angle p1 p2))) -1e-14)
  )

  ;; 3d-coord->pt-lst Convertit une liste de coordonnes 3D en liste de points
  (defun 3d-coord->pt-lst (lst)
    (if lst
      (cons (list (car lst) (cadr lst) (caddr lst))
            (3d-coord->pt-lst (cdddr lst))
      )
    )
  )

  ;; Triang2dArea retourne l'aire du triangle (points 2d)
  (defun Triang2dArea (p1 p2 p3)
    (/ (- (* (- (car p2) (car p1)) (- (cadr p3) (cadr p1)))
          (* (- (car p3) (car p1)) (- (cadr p2) (cadr p1)))
       )
       2.
    )
  )

  ;; extrusion et section des faces
  (defun loft (p1 p2 p3 / pline reg solid tmp)
    (if (clockwise-p p1 p2 p3)
      (setq tmp p2
            p2  p3
            p3  tmp
      )
    )
    (setq height (max (caddr p1) (caddr p2) (caddr p3))
          pline  (vlax-invoke
                   space
                   'AddLightweightPolyline
                   (list (car p1)
                         (cadr p1)
                         (car p2)
                         (cadr p2)
                         (car p3)
                         (cadr p3)
                   )
                 )
    )
    (vla-put-Closed pline :vlax-true)
    (vla-put-Elevation pline alt)
    (setq reg   (car (vlax-invoke space 'AddRegion (list pline)))
          solid (vla-AddExtrudedSolid space reg (- height alt) 0.0)
    )
    (if (not (and (equal (caddr p1) (caddr p2) 1e-9)
                  (equal (caddr p1) (caddr p3) 1e-9)
             )
        )
      (vla-SliceSolid
        solid
        (vlax-3d-point p1)
        (vlax-3d-point p2)
        (vlax-3d-point p3)
        :vlax-false
      )
    )
    (vla-Delete reg)
    (vla-Delete pline)
    (vla-update solid)
    solid
  )

  ;; fonction principale
  (or *acdoc*
      (setq *acdoc* (vla-get-ActiveDocument (vlax-get-acad-object)))
  )
  (setq space (vla-get-ModelSpace *acdoc*)
        shist (getvar 'solidhist)
  )
  (if (setq ss (ssget '((0 . "3DFACE"))))
    (progn
      (initget 6)
      (setq alt
             (cond
               ((getdist
                  "\nEntrez la distance sous le point le plus bas <1>: "
                )
               )
               (1.)
             )
      )
      (initget 1 "Auto Manuel")
      (setq opt (getkword "\nUnion des solides [Auto/Manuelle]: "))
      (vla-StartUndoMark *acdoc*)
      (setvar 'solidhist 0)
      (vlax-for f (setq ss (vla-get-ActiveSelectionSet *acdoc*))
        (setq coords (3d-coord->pt-lst (vlax-get f 'Coordinates))
              p1     (car coords)
              p2     (cadr coords)
              p3     (caddr coords)
              p4     (cadddr coords)
              lowest (if lowest
                       (apply 'min (cons lowest (mapcar 'caddr coords)))
                       (apply 'min (mapcar 'caddr coords))
                     )
        )
        (or (< (abs (Triang2dArea p1 p2 p3)) 1e-9)
            (setq flst (cons (list p1 p2 p3) flst))
        )
        (or (< (abs (Triang2dArea p3 p4 p1)) 1e-9)
            (setq flst (cons (list p3 p4 p1) flst))
        )
      )
      (vla-delete ss)
      (setq alt (- lowest alt))
      (and acet-ui-progress
           (acet-ui-progress "Extrusion..." (length flst))
      )
      (setq n 0)
      (foreach l flst
        (setq slst (cons (apply 'loft l) slst))
        (and acet-ui-progress (acet-ui-progress (setq n (1+ n))))
      )
      (and acet-ui-progress (acet-ui-progress))
      (if (= opt "Auto")
        (progn
          (and acet-ui-progress
               (acet-ui-progress "Union..." (length slst))
          )
          (setq i 0)
          (while (< 1 (setq cnt (length slst)))
            (if (= 1 (rem cnt 2))
              (progn
                (vla-Boolean (cadr slst) acUnion (car slst))
                (and acet-ui-progress
                     (acet-ui-progress (setq i (1+ i)))
                )
                (vla-update (cadr slst))
                (setq slst (cdr slst)
                      cnt  (1- cnt)
                )
              )
            )
            (setq n   0
                  tmp nil
            )
            (repeat (/ cnt 2)
              (vla-Boolean (nth n slst) acUnion (nth (1+ n) slst))
              (and acet-ui-progress (acet-ui-progress (setq i (1+ i))))
              (vla-update (nth n slst))
              (setq tmp (cons (nth n slst) tmp)
                    n   (+ 2 n)
              )
            )
            (setq slst tmp)
          )
          (and acet-ui-progress (acet-ui-progress))
        )
      )
      (setvar 'solidhist shist)
      (vla-EndUndoMark *acdoc*)
    )
  )
  (princ)
)