;;; 3d-coord->pt-lst Convertit une liste de coordonnes 3D en liste de points

;;; (3d-coord->pt-lst '(1.0 2.0 3.0 4.0 5.0 6.0)) -> ((1.0 2.0 3.0) (4.0 5.0 6.0))


(defun 3d-coord->pt-lst (lst)

  (if lst

    (cons (list (car lst) (cadr lst) (caddr lst))

          (3d-coord->pt-lst (cdddr lst))

    )

  )

)


;;; ACOS Retourne l'arc cosinus du nombre, en radians


(defun acos (num)

  (cond

    ((equal num 1 1e-9) 0.0)

    ((equal num -1 1e-9) pi)

    ((< -1 num 1)

     (atan (sqrt (- 1 (expt num 2))) num)

    )

  )

)


(defun c:clean_3dpoly (/ acdoc pl tol old new d1 d2 d3)

  (vl-load-com)

  (if

    (and

      (setq pl (car (entsel "\nSlectionnez une polyligne 3d: ")))

      (setq pl (vlax-ename->vla-object pl))

      (= (vla-get-ObjectName pl) "AcDb3dPolyline")

      (or (setq tol

                 (getdist

                   "\nSpcifiez la tolrance <0.0>: "

                 )

          )

          (setq tol 1e-9)

      )

    )

     (progn

       (vla-StartUndomark

         (vla-get-ActiveDocument (vlax-get-acad-object))

       )

       (setq old (3d-coord->pt-lst (vlax-get pl 'Coordinates)))

       (and (= (vla-get-Closed pl) :vlax-true)

            (setq old (append old (list (car old))))

       )

       (setq new (list (car old))

             old (cdr old)

       )

       (while (cadr old)

         (setq d1 (distance (car new) (car old))

               d2 (distance (car new) (cadr old))

               d3 (distance (car old) (cadr old))

         )

         (if (or

               (< d1 tol)

               (< d3 tol)

               (< (* d1

                     (sin (acos (/ (+ (* d1 d1) (* d2 d2) (- (* d3 d3)))

                                   (* 2 d1 d2)

                                )

                          )

                     )

                  )

                  tol

               )

             )

           (setq new (cons (car old) new)

                 old (cdr old)

           )

           (setq old (cdr old))

         )

       )

       (and (= (vla-get-Closed pl) :vlax-true)

            (setq old (list (car old)))

       )

       (vlax-put pl

                 'Coordinates

                 (apply 'append (append (reverse new) old))

       )

       (vla-EndUndomark

         (vla-get-ActiveDocument (vlax-get-acad-object))

       )

     )

  )

  (princ)

) 