;;; CONCENTRIC (gile)

;;; Cre des raccords concentriques sur des segments de polylignes parallles


(defun c:concentric (/       *error* HighlightSegment        getSegment

                     rad     p1      p2      ss1     vdir    lst1

                     p3      ss2     lst1    seg1    seg2    a

                     b       c       d       s       ang     cen

                    )

  (vl-load-com)


  (setq rad (getvar "FILLETRAD"))


;;;*************************************************************;;;


  (defun *error* (msg)

    (or (= msg "Fonction annule")

        (princ (strcat "Erreur : " msg))

    )

    (setvar "FILLETRAD" rad)

    (redraw)

    (princ)

  )


;;;*************************************************************;;;


  (defun HighlightSegment (pl par / p1 p2 n lst)

    (and

      (setq p1 (vlax-curve-getPointAtParam pl par))

      (setq p1 (trans p1 0 1))

      (setq p2 (vlax-curve-getPointAtParam pl (+ par 1)))

      (setq p2 (trans p2 0 1))

      (grvecs (list -255 p1 p2))

    )

  )


;;;*************************************************************;;;


  (defun getSegment (pl pt / pa p1 p2)

    (if

      (and

        (setq pa (fix (vlax-curve-getParamAtPoint pl pt)))

        (setq p1 (vlax-curve-getPointAtParam pl pa))

        (setq p2 (vlax-curve-getPointAtParam pl (1+ pa)))

      )

       (list p1 p2)

    )

  )


;;;*************************************************************;;;


  (princ (strcat "\nParamtre courant\tRayon = " (rtos rad)))

  (while (not (vl-consp p1))

    (initget 1 "Rayon")

    (setq

      p1 (getpoint

           "\nCliquez un point  l'intrieur du coude ou [Rayon]: "

         )

    )

    (if (= p1 "Rayon")

      (if (setq

            rad (getdist (strcat "\nSpcifiez le rayon du raccord <"

                                 (rtos rad)

                                 ">: "

                         )

                )

          )

        (setvar "FILLETRAD" rad)

        (setq rad (getvar "FILLETRAD"))

      )

    )

  )

  (if

    (and

      (setq p2 (getpoint p1 "\nSlectionnez une branche (Trajet): "))

      (setq ss1 (ssget "_F" (list p1 p2) '((0 . "LWPOLYLINE"))))

    )

     (progn

       (setq vdir (getvar "VIEWDIR"))

       (setq lst1

              (mapcar

                (function

                  (lambda (x)

                    (list (cadr x)

                          (vlax-curve-getClosestPointToProjection

                            (cadr x)

                            (cadr (cadddr x))

                            vdir

                          )

                    )

                  )

                )

                (ssnamex ss1)

              )

       )

       (foreach n lst1

         (HighlightSegment

           (car n)

           (fix (vlax-curve-getParamAtPoint (car n) (cadr n)))

         )

       )

       (if

         (and

           (setq p3

                  (getpoint p1 "\nSlectionnez l'autre branche (Trajet): ")

           )

           (setq ss2 (ssget "_F" (list p1 p3) '((0 . "LWPOLYLINE"))))

           (setq vdir (getvar "VIEWDIR"))

           (setq lst2

                  (mapcar

                    (function

                      (lambda (x)

                        (list (cadr x)

                              (vlax-curve-getClosestPointToProjection

                                (cadr x)

                                (cadr (cadddr x))

                                vdir

                              )

                        )

                      )

                    )

                    (ssnamex ss2)

                  )

           )

           (setq lst1 (vl-remove-if-not

                        (function

                          (lambda (x)

                            (assoc (car x) lst2)

                          )

                        )

                        lst1

                      )

           )

         )

          (progn

            (redraw)

            (setq seg1 (getsegment (caar lst1) (cadar lst1))

                  seg2 (getsegment (caar lst2) (cadar lst2))

            )

            (mapcar

              (function (lambda (s v) (set s v)))

              '(a b c d)

              (append seg1 seg2)

            )

            (if (setq s (inters a b c d nil))

              (progn

                (setq ang (/ (ang<2pi

                               (if (clockwise-p a s d)

                                 (- (angle d c) (angle a b))

                                 (- (angle a b) (angle d c))

                               )

                             )

                             2.0

                          )

                      cen (polar s

                                 (/ (+ (angle s a) (angle s d)) 2.0)

                                 (/ rad (sin ang))

                          )

                )

                (vl-cmdf "_.fillet"

                         "_nea"

                         (trans (cadar lst1) 0 1)

                         "_nea"

                         (trans (cadar lst2) 0 1)

                )

                (foreach n (cdr lst1)

                  (if

                    (and

                      (apply 'parallel-p

                             (append (getSegment (car n) (cadr n))

                                     seg1

                             )

                      )

                      (apply 'parallel-p

                             (append (getSegment

                                       (car n)

                                       (cadr (assoc (car n) lst2))

                                     )

                                     seg2

                             )

                      )

                    )

                     (progn

                       (setvar

                         "FILLETRAD"

                         (distance

                           cen

                           (vlax-curve-getClosestPointTo (car n) cen)

                         )

                       )

                       (vl-cmdf

                         "_.fillet"

                         "_nea"

                         (trans (cadr n) 0 1)

                         "_nea"

                         (trans (cadr (assoc (car n) lst2)) 0 1)

                       )

                     )

                  )

                )

                (setvar "FILLETRAD" rad)

              )

              (princ "\nSegments parallles.")

            )

          )

       )

     )

     (princ "\nAucune polyligne slectionne.")

  )

  (princ)

)


;;;********************** SOUS ROUTINES ************************;;;


;; PARALLELP

;; Retourne T si les segments p1 p2 et p3 p4 sont parallles

;;

;; Arguments : quatre points


(defun parallel-p (p1 p2 p3 p4)

  (equal '(0 0 0)

         (v^v (mapcar '- p1 p2) (mapcar '- p3 p4))

         1e-9

  )

)


;;;*************************************************************;;;


;; V^V

;; Retourne le produit vectoriel (vecteur) de deux vecteurs

;;

;; Arguments : deux vecteurs


(defun v^v (v1 v2)

  (list (- (* (cadr v1) (caddr v2)) (* (caddr v1) (cadr v2)))

        (- (* (caddr v1) (car v2)) (* (car v1) (caddr v2)))

        (- (* (car v1) (cadr v2)) (* (cadr v1) (car v2)))

  )

)


;;;*************************************************************;;;


;;; Clockwise-p

;;; Retourne T si les points p1 p2 et p3 tournent dans le sens horaire


(defun clockwise-p (p1 p2 p3)

  (< (sin (- (angle p1 p3) (angle p1 p2))) -1e-14)

)


;;;*************************************************************;;;


;;; Ang<2pi

;;; Retourne l'angle,  2*k*pi prs, compris entre 0 et 2*pi


(defun ang<2pi (ang)

  (if (and (<= 0 ang) (< ang (* 2 pi)))

    ang

    (ang<2pi (rem (+ ang (* 2 pi)) (* 2 pi)))

  )

) 